<?php 

namespace StripeSimple;

use App\Mailer as AppMailer;

class Mailer extends AppMailer
{
    protected $paymentIntent;

    public function setPaymentIntent($paymentIntent)
    {
        $this->paymentIntent = $paymentIntent;
    }

    public function paymentIntent()
    {
        return $this->paymentIntent;
    }

    protected function checkProperties()
    {
        if (!isset($this->config)) {
            throw new \LogicException('Config must be set.');
        }
        if (!isset($this->paymentIntent)) {
            throw new \LogicException('PaymentIntent must be set.');
        }
        if (!isset($this->view)) {
            throw new \LogicException('View must be set.');
        }
    }

    protected function getMailSettings()
    {
        $plugin = \App::getPlugin('StripeSimple');

        $settings = $this->config->getMailSettings();
        // extract admin setting
        $setting = $settings[0]; 
        unset($setting['replyTo']);
        unset($setting['replyToColumn']);
        unset($setting['replyToUser']);
        $setting['includeOrderId'] = false;

        if($this->paymentIntent->status === 'succeeded') {
            $setting['subject'] = $plugin->config['email_subject_konbini_succeeded'];
            $setting['template'] = '_mail_konbini_succeeded.txt';
        }
        else {
            $setting['subject'] = $plugin->config['email_subject_konbini_failed'];
            $setting['template'] = '_mail_konbini_failed.txt';
        }

        return [$setting];
    }

    public function send()
    {
        $this->checkProperties();
        $mailSettings = $this->getMailSettings();
        foreach ($mailSettings as $settings) {
            $message = $this->createMessageWithSettings($settings);
            $this->sendMessage($message);
        }
    }

    protected function getTemplateParams()
    {
        $data = [
            'payment_intent' => $this->paymentIntent,
            'order_id' => $this->paymentIntent->metadata->order_id ?: '',
            'user_mail' => $this->paymentIntent->receipt_email ?: '',
            'amount' => $this->paymentIntent->amount ?: 0,
        ];
        return $data; 
    }

    public function logOrder($message)
    {
        // DO NOTHING
    }

    protected function getSubject($settings, $params)
    {
        if (!isset($settings['subject'])) {
            throw new \UnexpectedValueException('Missing argument - subject');
        }
        $subject = $this->view->fetch('string:'.$settings['subject'], $params);
        return $subject;
    }
}
