<?php 

namespace StripeSimple;

use App\Controller;
use Stripe\PaymentMethod;

class WebhookController extends Controller
{
    public $plugin;

    public function __construct($type=null, $mode=null)
    {
        if(is_null($type)) {
            $type = 'webhook';
        }

        \Stripe\Stripe::setApiKey($this->plugin()->config['secretApiKey']);

        parent::__construct($type, $mode);
    }

    public function plugin()
    {
        if(!$this->plugin) {
            $this->plugin = \App::getPlugin('StripeSimple');
        }
        return $this->plugin;
    }

    public function process()
    {
        $event = null;
        $payload = \App::request()->getBody();
        if(!$payload) {
            return $this->json(['message' => 'empty payload.'], 400);
        }

        // \App::logger()->debug('payload: ' . $payload);

        try {
            $event = \Stripe\Event::constructFrom(json_decode($payload, true));
        }
        catch(\UnexpectedValueException $ex) {
            return $this->json(['message' => 'failed to parse request.'], 400);
        }
       
        if($this->plugin()->config['endpointSecret']) {
            $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
            try {
                $event = \Stripe\Webhook::constructEvent(
                  $payload, $sig_header, $this->plugin()->config['endpointSecret']
                );
            } catch(\Stripe\Exception\SignatureVerificationException $e) {
                return $this->json(['message' => 'failed to validate signature.'], 400);
            } 
        }
                
        // \App::logger()->debug('event: ' . print_r($event, true));

        switch($event->type) {
            case 'payment_intent.succeeded':
                // Fall through
            case 'payment_intent.payment_failed':
                $paymentIntent = $event->data->object;
                \App::logger()->debug($event->type . ': ' . print_r($paymentIntent, true));
                if($this->isSkeletonCartOrder($paymentIntent)
                    && $this->isKonbiniPayment($paymentIntent)) {
                    $this->sendMail($paymentIntent);
                }
                break;
            default:
                \App::logger()->debug('Unknown event received:' . $event->type);
                break;
        }
        
        return $this->json(['message' => 'success.'], 200);
    }

    protected function isSkeletonCartOrder($paymentIntent)
    {
        return isset($paymentIntent->metadata->client) && $paymentIntent->metadata->client === 'skeletoncart';
    }

    protected function isKonbiniPayment($paymentIntent)
    {
        return array_search('konbini', $paymentIntent->payment_method_types) !== false;
    }

    protected function isPaymentExpired($paymentIntent)
    {
        return $paymentIntent->status === 'requires_payment_method'
            && $paymentIntent->last_payment_error
            && $paymentIntent->last_payment_error->code === 'payment_intent_payment_attempt_expired';
    }

    public function sendMail($paymentIntent)
    {
        $mailer = new \StripeSimple\Mailer();
        $mailer->setConfig(\App::config());
        $mailer->setView(\App::view());
        $mailer->setPaymentIntent($paymentIntent);
        $mailer->send();
    }
}