(function($) {
    var settings = $.extend({
        apiKey: '',
        clientSecret: ''
    }, typeof StripeSimpleSettings === 'undefined' ? {} : StripeSimpleSettings);
    var stripe = Stripe(settings.apiKey, { locale: 'ja' });
    var options = {
        clientSecret: settings.clientSecret,
        appearance: { theme: 'flat' }
    };
    var elements = stripe.elements(options);
    var paymentElement = elements.create("payment");
    paymentElement.mount('#payment-element');
    paymentElement.addEventListener('change', function(event) {
        var $displayError = $('#card-errors');
        if(event.error) {
            $displayError.text(event.error.message);
        }
        else {
            $displayError.text('');
        }
        // console.log('element changed: ', event);
        if(event.value.type === 'card') {
            $('#about-cvc').show();
        }
        else {
            $('#about-cvc').hide();
        }
    });
    var $form = $('#payment-form');
    $form.submit(function(event) {
        event.preventDefault();
        showLoading(true);
        stripe.confirmPayment({
            elements,
            confirmParams: {
                return_url: '',
                payment_method_data: {
                    billing_details: {
                        email: settings.userMail,
                        name: settings.userName,
                        phone: settings.userTel
                    }
                }
            },
            redirect: 'if_required'
        }).then(function(result) {
            // console.log('then:', result);
            if(result.error) {
                console.log(result.error.message);
                $('#payment-result-error').removeClass('d-none');
                $('#payment-result-error-message').text(result.error.message);
                hideError(10000);
            }
            else {
                // console.log('paymentIntent:', result.paymentIntent);
                if(result.paymentIntent.status === 'succeeded') {
                    // console.log('Payment succeeded');
                    $('#payment-form').addClass('d-none');
                    $('.payment-way-text').addClass('d-none');
                    $('.payment-result-success').removeClass('d-none');
                }
                else if(result.paymentIntent.status === 'requires_action'
                    || result.paymentIntent.status === 'requires_source_action') {
                    // console.log('Payment requires action...');
                    var details = result.paymentIntent.next_action.konbini_display_details;
                    if(details) {
                        $('#payment-form').addClass('d-none');
                        $('.payment-way-text').addClass('d-none');
                        $('.payment-result-success').removeClass('d-none');
                        $('.payment-result-success-konbini').removeClass('d-none');
                        $('.payment-result-success-konbini a').prop('href', details.hosted_voucher_url);
                        var d = new Date(details.expires_at * 1000);
                        $('.payment-result-success-konbini span.payment-expires').text(d.toLocaleDateString());
                    }
                }
                else {
                    console.log('Unknown status, Something wrong...');
                }
            }
            showLoading(false);
        }).catch(function(error) {
            console.log('Error catch: ', error);
        }).finally(function() {
            // console.log('finally:');
        });
    });
    var showLoading = function(state) {
        if(state) {
            $('#submit').prop('disabled', true);
            $('#submit span.spinner-border').removeClass('d-none');
            $('#submit span.sr-only').removeClass('d-none');
        }
        else {
            $('#submit').prop('disabled', false);
            $('#submit span.spinner-border').addClass('d-none');
            $('#submit span.sr-only').addClass('d-none');
        }
    };
    var hideError = function(timeout) {
        setTimeout(function() {
            $('#payment-result-error').addClass('d-none');
            $('#payment-result-error-message').text('');
        }, timeout);
    };
})(jQuery);
